// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
//
// Copyright  1993, 1994  Microsoft Corporation.  All Rights Reserved.
//
//   PROGRAM:   RTFTB.c
//
//   PURPOSE:   RTF template for Windows applications
//   
//   
//   PLATFORMS: Chicago,NT
//   
//   FUNCTIONS:
//      WinMain() - calls initialization function, processes message loop
//      InitApplication() - initializes window data and registers window
//      InitInstance() - saves instance handle and creates main window
//      WndProc() - processes messages
//      CenterWindow() - used to center the "About" box over application window
//      About() - processes messages for "About" dialog box
//   
//   COMMENTS:
//   
//      The Windows SDK RTF Application Example is a sample application
//      that you can use to get an idea of how to perform some of the simple
//      functionality that all Applications written for Microsoft Windows
//      should implement. You can use this application as either a starting
//      point from which to build your own applications, or for quickly
//      testing out functionality of an interesting Windows API.
//      
//      This application is source compatible for with Windows 3.1 and
//      Windows NT.
//      
//   SPECIAL INSTRUCTIONS: N/A
//   

#include <windows.h>                // required for all Windows applications
#if !defined(_WIN32)
#include <ver.h>
#endif
#include "RTFTB.h"                // specific to this program

// Windows NT defines APIENTRY, but 3.x doesn't
#if !defined (APIENTRY)
#define APIENTRY far pascal
#endif

// Windows 3.x uses a FARPROC for dialogs
#if !defined(_WIN32)
#define DLGPROC FARPROC
#endif

HINSTANCE hInst;                     // current instance

char szAppName[] = "RTFTB";        // The name of this application
char szTitle[]   = "RTF Sample Application"; // The title bar text

//*********************** NEW CODE START *******************

#include <commctrl.h>

void TOOLTIP_GetText ( LPTOOLTIPTEXT lpttt );
void STATUS_DisplayStatusMenuHelp ( HWND hStatusWnd, UINT idMsg );
void STATUS_DisplayStatusInformation ( HWND hRTFWnd, HWND hStatusWnd );

//*********************** NEW CODE END *******************

#include <richedit.h>

int  RTF_GetCharAttribute ( HWND hRTFWnd, DWORD dwMask, DWORD dwEffects );
void RTF_ChangeCharAttribute ( HWND hRTFWnd, DWORD dwMask, DWORD dwEffects );
void RTF_ChangeSizeAttribute ( HWND hRTFWnd, int iPointChange );
void RTF_ChangeFont ( HWND hWnd, HWND hRTFWnd );
BOOL RTF_Open ( HWND hWnd, HWND hRTFWnd );
BOOL RTF_Save ( HWND hWnd, HWND hRTFWnd );
BOOL RTF_CheckForDirtyContents ( HWND hWnd, HWND hRTFWnd );
BOOL SelectPrinter ( LPSTR szPrinter, 
                     HWND  hWndParent, 
                     UINT  FAR *pdmDriverExtra );
HDC  GetPrinterDC ( LPSTR szPrinter, 
                    UINT  dmDriverExtra, 
                    HWND  hWndParent );
void RTF_Print ( HWND hWnd, HWND hRTFWnd );
void RTF_UpdateMenuState ( HWND hWndRTF, HWND hWnd );
void RTF_GetCurrentFont ( HWND hRTFWnd, LPSTR szFont );
void RTF_SetCurrentFont ( HWND hRTFWnd, LPSTR szFont );


UINT     dmDriverExtra;    // Used for GetPrinterDC
TCHAR    szPrinter[256];   // Used for GetPrinterDC
HANDLE   hRTFLib;          // Handle to RICHED32.DLL
HWND     hRTFWnd;          // Handle to RTF Edit Control
TCHAR    szFileName[256];  // Name of currently open file

//*********************** NEW CODE START *******************

HWND     hTBWnd;           // Handle of Toolbr Window
HWND     hStatusWnd;       // Handle of Status Bar Window
HWND     hComboBox;        // Handle of Combobox on toolbar
HBITMAP  hTBBmp;           // Bitmap to use for Toolbar

void Handle_WM_MENSELECT ( HWND hStatusWnd, UINT message, WPARAM uParam, LPARAM lParam );

//*********************** NEW CODE END *******************

// These are app-specific codes to determine the state of text.
// YES means the text has the attribure requested. No means it
// doesn't. WAFFLE means it is neither yes or no (has characters
// with and without the attribute requested).

#define CHAR_ATTRIB_YES    0
#define CHAR_ATTRIB_NO     1
#define CHAR_ATTRIB_WAFFLE 2

//
//   FUNCTION: WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
//
//   PURPOSE: calls initialization function, processes message loop
//   
//   COMMENTS:
//   
//      Windows recognizes this function by name as the initial entry point
//      for the program.  This function calls the application initialization
//      routine, if no other instance of the program is running, and always
//      calls the instance initialization routine.  It then executes a 
//      message retrieval and dispatch loop that is the top-level control 
//      structure for the remainder of execution.  The loop is terminated 
//      when a WM_QUIT	message is received, at which time this function 
//      exits the application instance by returning the value passed by 
//      PostQuitMessage().
//      
//      If this function must abort before entering the message loop, it
//      returns the conventional value NULL.
//

int APIENTRY WinMain(
               HINSTANCE hInstance,
               HINSTANCE hPrevInstance,
               LPSTR lpCmdLine,
               int nCmdShow
               )
{
   MSG msg;
   HANDLE hAccelTable;

   // Other instances of app running?
   if (!hPrevInstance) {
      // Initialize shared things 
      if (!InitApplication(hInstance)) {
         return (FALSE);               // Exits if unable to initialize
      }
   }

   // Perform initializations that apply to a specific instance 
   if (!InitInstance(hInstance, nCmdShow)) {
      return (FALSE);
   }

   hAccelTable = LoadAccelerators (hInstance, szAppName);

   // Acquire and dispatch messages until a WM_QUIT message is received.
   while (GetMessage(&msg,   // message structure
                     NULL,   // handle of window receiving the message
                     0,      // lowest message to examine
                     0)){    // highest message to examine
       if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg)) {
         TranslateMessage(&msg);// Translates virtual key codes
         DispatchMessage(&msg); // Dispatches message to window
       }
   }

   // Free RICHED32.DLL at shutdown

   if (hRTFLib)
     {
     FreeLibrary (hRTFLib);
     hRTFLib = NULL;
     }

   // Returns the value from PostQuitMessage
   return (msg.wParam);

   // This will prevent 'unused formal parameter' warnings
   lpCmdLine;
}


//
//   FUNCTION: InitApplication(HINSTANCE)
//
//   PURPOSE: Initializes window data and registers window class
//   
//   COMMENTS:
//   
//      This function is called at initialization time only if no other
//      instances of the application are running.  This function performs
//      initialization tasks that can be done once for any number of running
//      instances.
//      
//      In this case, we initialize a window class by filling out a data
//      structure of type WNDCLASS and calling the Windows RegisterClass()
//      function.  Since all instances of this application use the same 
//      window class, we only need to do this when the first instance is 
//      initialized.
//

BOOL InitApplication(HINSTANCE hInstance)
{
   WNDCLASS  wc;

   // Fill in window class structure with parameters that describe the
   // main window.
   wc.style         = CS_HREDRAW | CS_VREDRAW; // Class style(s).           
   wc.lpfnWndProc   = (WNDPROC)WndProc;        // Window Procedure          
   wc.cbClsExtra    = 0;                       // No per-class extra data.  
   wc.cbWndExtra    = 0;                       // No per-window extra data. 
   wc.hInstance     = hInstance;               // Owner of this class       
   wc.hIcon         = LoadIcon (hInstance, szAppName);// Icon name from .RC 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW); // Cursor                
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);// Default color             
   wc.lpszMenuName  = szAppName;               // Menu name from .RC        
   wc.lpszClassName = szAppName;               // Name to register as       

   // Register the window class and return success/failure code.
   return (RegisterClass(&wc));
}

//*********************** NEW CODE START *******************

// These macros add buttons/separators to the toolbar

#define ADDBUTTON( id, btntype, bmpos )                       \
{                                                             \
   tbb.fsStyle   = btntype;                                   \
   tbb.idCommand = id;                                        \
   tbb.iBitmap   = bmpos;                                     \
   tbb.iString   = bmpos;                                     \
   SendMessage ( hTBWnd, TB_INSERTBUTTON, 99, (LPARAM)&tbb ); \
}

#define ADDSEP( width )                                       \
{                                                             \
   tbb.fsStyle   = TBSTYLE_SEP;                               \
   tbb.idCommand = 0;                                         \
   tbb.iBitmap   = width;                                     \
   tbb.iString   = 0;                                         \
   SendMessage ( hTBWnd, TB_INSERTBUTTON, 99, (LPARAM)&tbb ); \
}

//**********************************************************
//
//  This procedure is called by the EnumFontFamilies
//  call. It fills the combobox with the font facename
//
//**********************************************************

int CALLBACK FillFontProc ( LPLOGFONT    lplf, 
                            LPTEXTMETRIC lptm,
                            int          iFontType,
                            LPARAM       lParam
                          )
{
  // Only add the facename if the facename is not
  // in the combobox

  if (CB_ERR == 
      (WPARAM)SendMessage ( (HWND)lParam, 
                            CB_FINDSTRING,
                            (WPARAM)-1,
                            (LPARAM)(lplf->lfFaceName)
                          )
     )
    SendMessage ( (HWND)lParam,  
                  CB_ADDSTRING,
                  (WPARAM)0,
                  (LPARAM)(lplf->lfFaceName)
                );
   return TRUE;
}

//**********************************************************
//
// TOOLBAR_MakeMeAToolbar: This code creates a toolbar and
//                         sticks buttons on it
//
//**********************************************************

HWND TOOLBAR_MakeMeAToolbar ( HWND hWndParent )
{
   TBADDBITMAP32 tbadbm;  // For adding the bitmap
   TBBUTTON      tbb;     // For adding buttons
   HWND          hTBWnd;  // Handle of toolbar window
   HDC           hDC;     // a DC

   // Create the toolbar window with CreateWindow

   hTBWnd = CreateWindow(
      TOOLBARCLASSNAME,	       // Classname of toolbar
      "",                      // No text
      WS_CHILD | WS_VISIBLE | 
            TBSTYLE_TOOLTIPS,  // Style of toolbar
      0, 0, 0, 0,              // Will size to fit in parent's WM_SIZE code
      hWndParent,              // Use main window as parent
      NULL,		                // Don't worry about the ID
      hInst,    	             // This instance owns this window.       
      NULL		                // Always NULL for RTF Edit windows
      );

   if (!hTBWnd) return NULL;

   // This is version control-- by sending the size of the TBBUTTON
   // structure, the toolbar knows what version of the toolbar the program
   // expects to be talking to
   SendMessage ( hTBWnd, TB_BUTTONSTRUCTSIZE, sizeof(TBBUTTON), 0 );

   // Let's use 32x32 buttons
   SendMessage ( hTBWnd, TB_SETBUTTONSIZE, 0, MAKELONG(32,32));

   // Grab the toolbar bitmap from the resources
   hTBBmp = LoadBitmap ( hInst, "TB" );

   // And instruct the toolbar to use it
   tbadbm.hInst = NULL;
   tbadbm.nID   = (UINT)hTBBmp;
   SendMessage ( hTBWnd, TB_ADDBITMAP32, 10, (LPARAM)&tbadbm );

   // Add a combobox to the toolbar. This involves three steps:
   // 
   // 1. Create the combobox, using the main window as a parent
   // 2. Changing the parent to the toolbar
   // 3. Sizing the control
   // 
   // The reason the parent must originally be the main window
   // is so we can get the CBN_SELCHANGE messages. Controls
   // cache away their parent upon creation. Therefore, even though
   // we have re-parented the combobox to the toolbar, the CBN_*
   // messages still go to the orignal parent. This allows us to
   // avoid subclassing the toolbar to look for the CBN_* messages.

   hComboBox = CreateWindow ( "combobox",
                              "",
                              WS_CHILD | WS_VISIBLE | CBS_SORT | CBS_DROPDOWNLIST | WS_VSCROLL,
                              0, 0, 0, 0,
                              hWndParent,
                              (HMENU)ID_COMBOBOX,
                              hInst,
                              NULL
                            );

   // Fill the combobox with fonts

   hDC = GetDC ( hWndParent );
   EnumFontFamilies ( hDC, NULL, FillFontProc, (LPARAM)hComboBox );
   ReleaseDC ( hWndParent, hDC );

   // Parent the combobox to the toolbar, and make it 200 pixels wide

   SetParent (hComboBox, hTBWnd );
   MoveWindow ( hComboBox, 10, 5, 200, 100, FALSE );

   // Set up the fields of the TBBUTTON structure that all the buttons
   // will have in common. The ADDBUTTON macro fills in the fields
   // that vary from button to button.
   tbb.fsState   = TBSTATE_ENABLED;
   tbb.dwData    = 0;

   // First, add a seperator that is 210 pixels wide, the combobox 
   // will occupy that space. The ADDSEP macro adds a button with 
   // TBSTYLE_SEP for the style, and the width goes in iBitmap.

   ADDSEP( 210 );

   // Add the buttons to the toolbar. The first parameter to the
   // ADDBUTTON macro indicates the ID, the second parameter indicates
   // the type of button, and the third indicates which part of the 
   // bitmap to use.

   ADDBUTTON( IDM_NEW,  TBSTYLE_BUTTON, 0 )
   ADDBUTTON( IDM_OPEN, TBSTYLE_BUTTON, 1 )
   ADDBUTTON( IDM_SAVE, TBSTYLE_BUTTON, 2 )

   ADDSEP( 0 ); // Add a "nice" width separator

   ADDBUTTON( IDM_PRINT, TBSTYLE_BUTTON, 3 )

   ADDSEP( 0 ); // Add a "nice" width separator

   ADDBUTTON( IDM_BOLD,      TBSTYLE_CHECK, 4 )
   ADDBUTTON( IDM_ITALIC,    TBSTYLE_CHECK, 5 )
   ADDBUTTON( IDM_UNDERLINE, TBSTYLE_CHECK, 6 )

   ADDSEP( 0 ); // Add a "nice" width separator

   ADDBUTTON( IDM_INCREASE_FONT, TBSTYLE_BUTTON, 7 )
   ADDBUTTON( IDM_DECREASE_FONT, TBSTYLE_BUTTON, 8 )
   ADDBUTTON( IDM_FONTDIALOG,    TBSTYLE_BUTTON, 9 )

   // Add the stringtable. The indexes of the strings for this example
   // mirror the indexes of the bitmaps, but that is specific to this
   // example program. Your program can use any index it wishes that
   // exists in the stringtable.

   SendMessage ( hTBWnd, TB_ADDSTRING, 0, 
                 (LPARAM)"New\0Open\0Save\0Print\0Bold\0Italic\0Underline\0Increase\0Decrease\0Fonts\0" );

   return hTBWnd;
}

//**********************************************************
//
// STATUS_MakeMeAStatusLine: This code creates a status line and
//                           sticks panes on it.
//
//**********************************************************

HWND STATUS_MakeMeAStatusLine ( HWND hWndParent )
{
  HWND hStatusWnd;

  // This array identifies the pane positions. The first pane
  // goes from 0 to 100, the second pane goes from a little bit
  // past the first pane to the right edge of the status line.

  int iPanesPosition[2] = { 100, -1 };

  // Create the status bar window using the nifty CreateStatusWindow
  // function (you culd also just use CreateWindow, but I thought
  // I would show you an alternate method.
  hStatusWnd = CreateStatusWindow
                 ( WS_CHILD | WS_VISIBLE, // Window style.                         
                   "Ready",               // Pane 0 text
                   hWndParent,            // Parent Window
                   4242                   // Some ID
                 );

  // Tell the status bar that there are two panes, and give the
  // array from above to indicate the pane sizes.
  SendMessage ( hStatusWnd, SB_SETPARTS, 2, (LPARAM)iPanesPosition );

  // Call the function that fills in the second pane with the
  // line # of the caret in the RTF edit control. This is so the user
  // gets the feedback immediately at startup.
  STATUS_DisplayStatusInformation ( hRTFWnd, hStatusWnd);

  return hStatusWnd;
}

//*********************** NEW CODE END *******************

//
//   FUNCTION:  InitInstance(HINSTANCE, int)
//
//   PURPOSE:  Saves instance handle and creates main window
//   
//   COMMENTS:
//   
//      This function is called at initialization time for every instance of
//      this application.  This function performs initialization tasks that
//      cannot be shared by multiple instances.
//      
//      In this case, we save the instance handle in a static variable and
//      create and display the main program window.
//

BOOL InitInstance(
         HINSTANCE     hInstance,
         int           nCmdShow
         )
{
   HWND    hWnd; // Main window handle.

   // Save the instance handle in static variable, which will be used in
   // many subsequence calls from this application to Windows.

   hInst = hInstance; // Store instance handle in our global variable

   // Create a main window for this application instance.
   hWnd = CreateWindow(
      szAppName,	        // See RegisterClass() call.             
      szTitle,	           // Text for window title bar.            
      WS_OVERLAPPEDWINDOW,// Window style.                         
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0,// Use default positioning
      NULL,		           // Overlapped windows have no parent.    
      NULL,		           // Use the window class menu.            
      hInstance,	        // This instance owns this window.       
      NULL		           // We don't use any data in our WM_CREATE
      );

   // If window could not be created, return "failure"
   if (!hWnd) {
      return (FALSE);
   }

   // The call to LoadLibrary automatically registers
   // the classname of the RTF Edit control
   hRTFLib = LoadLibrary ( "RICHED32.DLL" );
   if (!hRTFLib)
     {
     MessageBox ( NULL, "LoadLibrary on RICHED32.DLL Failed", "MSJ", MB_OK );
     return FALSE;
     }

   // Create the child RTF Edit Control
   hRTFWnd = CreateWindow(
      "RICHEDIT",	        // Classname of RTF Edit Control
      "",                 // Text for edit control
      WS_CHILD | WS_VISIBLE | ES_MULTILINE,// Window style.                         
      0, 0, 0, 0,         // Will size to fit in parent's WM_SIZE code
      hWnd,		           // Use main window as parent
      NULL,		           // Don't worry about the ID
      hInstance,	        // This instance owns this window.       
      NULL		           // Always NULL for RTF Edit windows
      );

   // Instruct the RTF Edit control to notify the parent whenever
   // the selection in the RTF Edit control changes. This will cause
   // the WM_NOTIFY:EN_SELCHANGE message to be sent to the parent.
   SendMessage (hRTFWnd, EM_SETEVENTMASK, 0, ENM_SELCHANGE );

//*********************** NEW CODE START *******************

   // Must be called to register the classes in COMCTL32.DLL
   InitCommonControls();

   // Build the toolbar and status bar
   hTBWnd     = TOOLBAR_MakeMeAToolbar ( hWnd );
   hStatusWnd = STATUS_MakeMeAStatusLine ( hWnd );

//*********************** NEW CODE END *******************

   // Make the window visible; update its client area; and return "success"
   ShowWindow(hWnd, nCmdShow); // Show the window        
   UpdateWindow(hWnd);         // Sends WM_PAINT message 

   return (TRUE);              // We succeeded...        
}

//
//   FUNCTION: WndProc(HWND, UINT, WPARAM, LPARAM)
//
//   PURPOSE:  Processes messages
//   
//   MESSAGES:
//   
//      WM_COMMAND    - application menu (About dialog box)
//      WM_DESTROY    - destroy window
//   
//   COMMENTS:
//   
//      To process the IDM_ABOUT message, call MakeProcInstance() to get the
//      current instance address of the About() function.  Then call Dialog
//      box which will create the box according to the information in your
//      RTFTB.rc file and turn control over to the About() function.  When
//      it returns, free the intance address.
//

#define ptrNMHDR       ((LPNMHDR)lParam)
#define ptrSELCHANGE   ((SELCHANGE *)lParam)

LRESULT CALLBACK WndProc(
      HWND hWnd,         // window handle
      UINT message,      // type of message
      WPARAM uParam,     // additional information
      LPARAM lParam      // additional information
      )
{
   FARPROC lpProcAbout;  // pointer to the "About" function
   int wmId, wmEvent;

   switch (message) {

//*********************** NEW CODE START *******************

      case WM_MENUSELECT:

           // Call the function to update the status bar
           Handle_WM_MENSELECT ( hStatusWnd, message, uParam, lParam );
           break;

      case WM_SIZE:

         // Sending the WM_SIZE message to the toolbar and status
         // bar makes them size themselves to appropriately fit in 
         // the client area of the parent.

         if ( hRTFWnd && hTBWnd && hStatusWnd)
           {
           RECT rcTB;
           RECT rcSB;

           // Force TB and SB to resize themselves
           SendMessage ( hTBWnd,     message, uParam, lParam );
           SendMessage ( hStatusWnd, message, uParam, lParam );

           // Figure out the rectangle region to put the RTF Edit control
           // in by subtracting out the toolbar and status bar
           GetWindowRect ( hTBWnd,     &rcTB );
           GetWindowRect ( hStatusWnd, &rcSB );
           MoveWindow ( hRTFWnd, 0, rcTB.bottom-rcTB.top, 
                        LOWORD (lParam), 
                        (HIWORD (lParam) - 
                        ((rcTB.bottom-rcTB.top) +
                        (rcSB.bottom-rcSB.top))
                      ), TRUE);
           }
         break;

      case WM_NOTIFY:

         switch (ptrNMHDR->code)
           {
           case EN_SELCHANGE:
             
             // Call our function to check/uncheck the menu
             // items corresponding to the bold, italic, and
             // underline states of the selected text.
             RTF_UpdateMenuState ( hRTFWnd, hWnd );

             // Call our function to display the row and column
             // in the status bar.
             STATUS_DisplayStatusInformation ( hRTFWnd, hStatusWnd);
             break;

           case TTN_NEEDTEXT:

             // This message is sent when the toolbar needs
             // tooltip text.
             TOOLTIP_GetText ( (LPTOOLTIPTEXT)lParam );
             break;

           default:

             return (DefWindowProc(hWnd, message, uParam, lParam));
           } // End Switch
        break;
 
//*********************** NEW CODE END *******************

      case WM_COMMAND:  // message: command from application menu

         // Message packing of uParam and lParam have changed for Win32,
         // let us handle the differences in a conditional compilation:
         #if defined (_WIN32)
             wmId    = LOWORD(uParam);
			    wmEvent = HIWORD(uParam);
         #else
            wmId    = uParam;
            wmEvent = HIWORD(lParam);
         #endif

         switch (wmId) {

//************************* NEW CODE START ************************   

            // This is sent when the user changes the selection in the
            // combobox on the toolbar

            case ID_COMBOBOX:

               if ( CBN_SELCHANGE == wmEvent ) 
                 {
                 // Get the facename from the combobox
                 char szFont[LF_FACESIZE];
                 GetWindowText ( hComboBox, szFont, sizeof(szFont));
                 // Set the new font in the RTF edit control
                 RTF_SetCurrentFont ( hRTFWnd, szFont );
                 }
               break;

//************************* NEW CODE END ************************   

            case IDM_ABOUT:
               lpProcAbout = MakeProcInstance((FARPROC)About, hInst);

               DialogBox(hInst,           // current instance         
                  "AboutBox",             // dlg resource to use      
                  hWnd,                   // parent handle            
                  (DLGPROC)lpProcAbout);  // About() instance address 

               FreeProcInstance(lpProcAbout);
               break;

            case IDM_EXIT:
               DestroyWindow (hWnd);
               break;

            case IDM_HELPCONTENTS:
               if (!WinHelp (hWnd, "RTFTB.HLP", HELP_KEY,(DWORD)(LPSTR)"CONTENTS")) {
                  MessageBox (GetFocus(),
                     "Unable to activate help",
                     szAppName, MB_SYSTEMMODAL|MB_OK|MB_ICONHAND);
				  
               }
               break;

            case IDM_HELPSEARCH:
               if (!WinHelp(hWnd, "RTFTB.HLP", HELP_PARTIALKEY, (DWORD)(LPSTR)"")) {
                  MessageBox (GetFocus(),
                     "Unable to activate help",
                     szAppName, MB_SYSTEMMODAL|MB_OK|MB_ICONHAND);
               }
               break;

            case IDM_HELPHELP:
               if(!WinHelp(hWnd, (LPSTR)NULL, HELP_HELPONHELP, 0)) {
                  MessageBox (GetFocus(),
                     "Unable to activate help",
                     szAppName, MB_SYSTEMMODAL|MB_OK|MB_ICONHAND);
               }
               break;

            // For the BOLD, ITALIC, and UNDERLINE menu options,
            // we change the character attribytes with our function,
            // and then update the menu to reflect the new state
            // of the current text.

            case IDM_BOLD:

               RTF_ChangeCharAttribute ( hRTFWnd, CFM_BOLD, CFE_BOLD );
               RTF_UpdateMenuState ( hRTFWnd, hWnd );
               break;

            case IDM_ITALIC:

               RTF_ChangeCharAttribute ( hRTFWnd, CFM_ITALIC, CFE_ITALIC );
               RTF_UpdateMenuState ( hRTFWnd, hWnd );
               break;

            case IDM_UNDERLINE:

               RTF_ChangeCharAttribute ( hRTFWnd, CFM_UNDERLINE, CFE_UNDERLINE );
               RTF_UpdateMenuState ( hRTFWnd, hWnd );
               break;

            case IDM_INCREASE_FONT:

               // Increase the font size by 2 points
               RTF_ChangeSizeAttribute ( hRTFWnd, 2 );
               break;

            case IDM_DECREASE_FONT:

               // Decrease the font size by 2 points
               RTF_ChangeSizeAttribute ( hRTFWnd, -2 );
               break;

            case IDM_FONTDIALOG:

               // Bring up the Font Common Dialog and apply that new
               // font to the selected text.
               RTF_ChangeFont ( hWnd, hRTFWnd );
               // Update the menu state to relfect the new font.
               RTF_UpdateMenuState ( hRTFWnd, hWnd );
               break;

            case IDM_NEW:

               // We check to see if the contents of the edit
               // control need to be saved with the RTF_CheckForDirtyContents
               // function. After that function has returned an acceptable
               // value, we will clear the contents of the edit control
               // by selecting the entire range and replacing it with an
               // empty string.

               if (RTF_CheckForDirtyContents ( hWnd, hRTFWnd ))
                 {
                 CHARRANGE cr;

                 // Reset the global filename to an empty string
                 *szFileName = 0;

                 // Set the character range to go from start to end.
                 cr.cpMin = 0;
                 cr.cpMax = -1;

                 // The WM_SETREDRAW game keeps screen flicker down.
                 SendMessage ( hRTFWnd, WM_SETREDRAW, FALSE, 0 );
                 SendMessage ( hRTFWnd, EM_EXSETSEL,   0, (LPARAM)&cr );
                 SendMessage ( hRTFWnd, WM_SETREDRAW, TRUE, 0 );
                 SendMessage ( hRTFWnd, EM_REPLACESEL, 0, (LPARAM)"" );

                 // Clear the dirty bit
                 SendMessage ( hRTFWnd, EM_SETMODIFY, FALSE, 0 );
                 }
               STATUS_DisplayStatusInformation ( hRTFWnd, hStatusWnd);
               break;

            case IDM_OPEN:
            
               // We check to see if the contents of the edit
               // control need to be saved with the RTF_CheckForDirtyContents
               // function. After that function has returned an acceptable
               // value, we will load a file and stream it into the
               // RTF Edit control.

               if (RTF_CheckForDirtyContents ( hWnd, hRTFWnd ))
                 {
                 // Stream in the new file
                 RTF_Open ( hWnd, hRTFWnd );
                 // Clear the dirty bit
                 SendMessage ( hRTFWnd, EM_SETMODIFY, FALSE, 0 );
                 // Update the status bar
                 STATUS_DisplayStatusInformation ( hRTFWnd, hStatusWnd);
                 }
               break;

            case IDM_SAVE:

               // Stream out the new file
               if (RTF_Save ( hWnd, hRTFWnd ))
                 {
                 // Clear the dirsty bit if save happened
                 SendMessage ( hRTFWnd, EM_SETMODIFY, FALSE, 0 );
                 }
               break;

            case IDM_SAVEAS:

               *szFileName = 0; // Forces Save As dialog to pop up
               // Stream out the new file
               if (RTF_Save ( hWnd, hRTFWnd ))
                 {
                 // Clear the dirsty bit if save happened
                 SendMessage ( hRTFWnd, EM_SETMODIFY, FALSE, 0 );
                 }
               break;

            case IDM_PRINTSETUP:
      
               // Standard common dialog fun
               SelectPrinter ( szPrinter, hWnd, &dmDriverExtra );
               break;

            case IDM_PRINT:
      
               // Print the contents of the RTF Edit Control
               RTF_Print ( hWnd, hRTFWnd );
               break;

            default:
               return (DefWindowProc(hWnd, message, uParam, lParam));

         }
         break;

      case WM_SETFOCUS:

         // When the fram gets focus, set focus to the RTF Edit control
         SetFocus ( hRTFWnd );
         break;

      case WM_CLOSE:

         // We check to see if the contents of the edit
         // control need to be saved with the RTF_CheckForDirtyContents
         // function. After that function has returned an acceptable
         // value, we shut down this app.

         if (RTF_CheckForDirtyContents ( hWnd, hRTFWnd ))
            DestroyWindow ( hWnd );
         break;

      case WM_DESTROY:  // message: window being destroyed

         DestroyWindow ( hTBWnd );
         DeleteObject ( hTBBmp );
         PostQuitMessage(0);
         break;

      default:          // Passes it on if unproccessed
         return (DefWindowProc(hWnd, message, uParam, lParam));
   }
   return (0);
}

//
//   FUNCTION: CenterWindow (HWND, HWND)
//
//   PURPOSE:  Center one window over another
//   
//   COMMENTS:
//   
//      Dialog boxes take on the screen position that they were designed 
//      at, which is not always appropriate. Centering the dialog over a 
//      particular window usually results in a better position.
//

BOOL CenterWindow (HWND hwndChild, HWND hwndParent)
{
   RECT    rChild, rParent;
   int     wChild, hChild, wParent, hParent;
   int     wScreen, hScreen, xNew, yNew;
   HDC     hdc;

   // Get the Height and Width of the child window
   GetWindowRect (hwndChild, &rChild);
   wChild = rChild.right - rChild.left;
   hChild = rChild.bottom - rChild.top;

   // Get the Height and Width of the parent window
   GetWindowRect (hwndParent, &rParent);
   wParent = rParent.right - rParent.left;
   hParent = rParent.bottom - rParent.top;

   // Get the display limits
   hdc = GetDC (hwndChild);
   wScreen = GetDeviceCaps (hdc, HORZRES);
   hScreen = GetDeviceCaps (hdc, VERTRES);
   ReleaseDC (hwndChild, hdc);

   // Calculate new X position, then adjust for screen
   xNew = rParent.left + ((wParent - wChild) /2);
   if (xNew < 0) {
      xNew = 0;
   }
   else if ((xNew+wChild) > wScreen) {
      xNew = wScreen - wChild;
   }

   // Calculate new Y position, then adjust for screen
   yNew = rParent.top  + ((hParent - hChild) /2);
   if (yNew < 0) {
      yNew = 0;
   }
   else if ((yNew+hChild) > hScreen) {
      yNew = hScreen - hChild;
   }

   // Set it, and return
   return SetWindowPos (hwndChild, NULL,
      xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}


//
//   FUNCTION: About(HWND, UINT, WPARAM, LPARAM)
//
//   PURPOSE:  Processes messages for "About" dialog box
//   
//   MESSAGES:
//   
//      WM_INITDIALOG - initialize dialog box
//      WM_COMMAND    - Input received
//   
//   COMMENTS:
//   
//      Display version information from the version section of the
//      application resource.
//      
//      Wait for user to click on "Ok" button, then close the dialog box.
//

LRESULT CALLBACK About(
      HWND hDlg,           // window handle of the dialog box
      UINT message,        // type of message
      WPARAM uParam,       // message-specific information
      LPARAM lParam
      )
{
   static  HFONT hfontDlg;

   switch (message) {
      case WM_INITDIALOG:  // message: initialize dialog box
         // Create a font to use
         hfontDlg = CreateFont(14, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0,
            VARIABLE_PITCH | FF_SWISS, "");

         // Center the dialog over the application window
         CenterWindow (hDlg, GetWindow (hDlg, GW_OWNER));

         return (TRUE);

      case WM_COMMAND:                      // message: received a command
         if (LOWORD(uParam) == IDOK         // "OK" box selected?
            || LOWORD(uParam) == IDCANCEL) {// System menu close command? 
            EndDialog(hDlg, TRUE);          // Exit the dialog
            DeleteObject (hfontDlg);
            return (TRUE);
         }
         break;
   }
   return (FALSE); // Didn't process the message

   lParam; // This will prevent 'unused formal parameter' warnings
}

/*************************************************************************

   The RTF_ChangeCharAttribute function will apply the character
   attributes to the selected text. The attributes used in this sample
   program are CFE_ITALIC, CFE_BOLD, and CFE_UNDERLINE. The mask
   corresponds to the attribute.

*************************************************************************/

void RTF_ChangeCharAttribute ( HWND hRTFWnd, DWORD dwMask, DWORD dwEffects )
{
    CHARFORMAT cf; // This is defined in RICHEDIT.H

    // Get the current status of the text into the CHARFORMAT structure,
    // use the selected text. wParam is TRUE to indicate the
    // selected textt, FALSE for the first character in the
    // RTF Edit control.
    SendMessage(hRTFWnd, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

    // Set the mask to tell the RTF Edit control to pay attention to
    // the appropriate bit of the dwEffects field.
    cf.dwMask      = dwMask;
    // Twiddle de bits, captain!
    cf.dwEffects  ^= dwEffects;

    // Set the new underline status to the selected text
    SendMessage(hRTFWnd, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
}

/*************************************************************************

   The RTF_GetCharAttribute function will indicate whether a selection
   of text has an attribute. The text can either have the attribute (YES),
   not have it (NO), or have some with it and some without it (WAFFLE).

*************************************************************************/

int RTF_GetCharAttribute ( HWND hRTFWnd, DWORD dwMask, DWORD dwEffects )
{
    CHARFORMAT cf; // This is defined in RICHEDIT.H

    // Get the current status of the text into the CHARFORMAT structure,
    // use the selected text. wParam is TRUE to indicate the
    // selected textt, FALSE for the first character in the
    // RTF Edit control.
    SendMessage(hRTFWnd, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

    if (cf.dwMask & dwMask)
      {
      if (cf.dwEffects & dwEffects)
        return CHAR_ATTRIB_YES;
      else
        return CHAR_ATTRIB_NO;
      }
    else
      {
      return CHAR_ATTRIB_WAFFLE;
      }
}

/*************************************************************************

   The RTF_ChangeSizeAttribute function will change the size of the selected
   text by the iPointChange value.

*************************************************************************/

void RTF_ChangeSizeAttribute ( HWND hRTFWnd, int iPointChange )
{
    CHARFORMAT cf; // This is defined in RICHEDIT.H

    // Get the current status of the text into the CHARFORMAT structure,
    // use the selected text. wParam is TRUE to indicate the
    // selected textt, FALSE for the first character in the
    // RTF Edit control.
    SendMessage(hRTFWnd, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

    // Set the mask to tell the RTF Edit control to pay attention to
    // the size bit of the dwEffects field.
    cf.dwMask      = CFM_SIZE;
    // Increase or decrease the font size, keeping it between 6 and 128 points
    if ( 
         ((cf.yHeight + 20*iPointChange) <= (128*20))
         &&
         ((cf.yHeight + 20*iPointChange) >= (6*20))
       )
      cf.yHeight += 20*iPointChange;

    // Set the new underline status to the selected text
    SendMessage(hRTFWnd, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
}

/*************************************************************************

   The RTF_ChangeFont function will use the font common dialog
   to change the font in the selected text.

*************************************************************************/

void RTF_ChangeFont ( HWND hWnd, HWND hRTFWnd )
{
    CHARFORMAT cf;                // This is defined in RICHEDIT.H
    CHOOSEFONT ChooseFontStruct;  // Common Dialog fun
    LOGFONT    lf;                // Log font information
    HDC        hDC;

    // Get the font formatting status into the CHARFORMAT strucuture,
    // use the selected text. wParam is TRUE to indicate the
    // selected text, FALSE for the first character in the
    // RTF Edit control.
    SendMessage(hRTFWnd, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

    // Fill in the font info for the font common fialog

    memset (&ChooseFontStruct, 0, sizeof(CHOOSEFONT));
    memset (&lf, 0, sizeof(LOGFONT));

    hDC = GetDC(hWnd); // Need a screen DC, use the parent's DC

    // The RTF Edit control measures in twips. Each point is
    // 20 twips.

    lf.lfHeight = cf.yHeight / -20;

    // Set up the rest of the logfont structure according to the 
    // information retrieved from the EM_GETCHARFORMAT message

    if (cf.dwEffects & CFE_BOLD)
        lf.lfWeight = FW_BOLD;
    else
        lf.lfWeight = FW_NORMAL;
    lf.lfItalic = (BOOL)(cf.dwEffects & CFE_ITALIC);
    lf.lfUnderline = (BOOL)(cf.dwEffects & CFE_UNDERLINE);
    lf.lfCharSet = DEFAULT_CHARSET;
    lf.lfQuality = DEFAULT_QUALITY;
    lf.lfPitchAndFamily = cf.bPitchAndFamily;
    lstrcpy(lf.lfFaceName, cf.szFaceName);

    // Fire up the common dialog.

    ChooseFontStruct.lStructSize = sizeof(ChooseFontStruct);
    ChooseFontStruct.hwndOwner = hWnd;
    ChooseFontStruct.hDC = hDC;
    ChooseFontStruct.lpLogFont = &lf;
    ChooseFontStruct.Flags = CF_SCREENFONTS | CF_INITTOLOGFONTSTRUCT;
    ChooseFontStruct.rgbColors = RGB(0,0,0);
    ChooseFontStruct.nFontType = SCREEN_FONTTYPE;

    if (ChooseFont(&ChooseFontStruct))
    {
        // Set the mask to tell the RTF Edit control to pay attention to
        // the font formatting bits.
        cf.dwMask      = CFM_BOLD | CFM_FACE | CFM_ITALIC | 
                         CFM_OFFSET | CFM_SIZE | CFM_UNDERLINE;

        //  Undo the equation from above.

        cf.yHeight = lf.lfHeight * -20;

        // Fill in the rest of the characater formatting
         
        cf.dwEffects = 0;
        if (FW_BOLD == lf.lfWeight)
            cf.dwEffects |= CFE_BOLD;
        if (lf.lfItalic)
            cf.dwEffects |= CFE_ITALIC;

        if (lf.lfUnderline)
            cf.dwEffects |= CFE_UNDERLINE;

        cf.bPitchAndFamily = lf.lfPitchAndFamily;

        lstrcpy (cf.szFaceName, lf.lfFaceName);

        // Set the new formatting to the selected text
        SendMessage(hRTFWnd, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
    }

    // Don't forget this!
    ReleaseDC(hWnd, hDC);
}

/*************************************************************************

   Quick and dirty function to get a filename. bOpen decides whether
   to use a File Open or a File Save As dialog.

*************************************************************************/

BOOL GetFileName ( HWND hWnd, LPSTR szFileName, BOOL bOpen )
{
  OPENFILENAME of;

  of.lStructSize       = sizeof(OPENFILENAME);
  of.hwndOwner         = (HWND)hWnd;
  of.hInstance         = (HANDLE)NULL;
  of.lpstrFilter       = "RTF Files (*.RTF)\0*.RTF\0";
  of.lpstrCustomFilter = (LPSTR)NULL;
  of.nMaxCustFilter    = 0L;
  of.nFilterIndex      = 1L;
  of.lpstrFile         = szFileName;
  of.nMaxFile          = 256;
  of.lpstrFileTitle    = NULL;
  of.nMaxFileTitle     = 0;
  of.lpstrInitialDir   = NULL;
  of.lpstrTitle        = (LPSTR)NULL;
  of.Flags             = OFN_HIDEREADONLY |
                         OFN_PATHMUSTEXIST ;
  of.nFileOffset       = 0;
  of.nFileExtension    = 0;
  of.lpstrDefExt       = (LPSTR)"RTF";
  of.lCustData         = 0L;
  of.lpfnHook          = 0L;  // Zero eliminates compiler warnings
  of.lpTemplateName    = (LPSTR)NULL;

  if ( bOpen )
    return GetOpenFileName ( &of );
  else
    return GetSaveFileName ( &of );
}

/*************************************************************************

   This is the callback stream used by the RTF Edit controls
   EM_STREAMOUT message. It is called repeatedly until the RTF Edit
   control has streamed all of the information.

*************************************************************************/

DWORD CALLBACK SaveCallback ( DWORD    dwCookie, // App supplied value
                              LPBYTE   pbBuff,   // Pointer to data
                              LONG     cb,       // size of data
                              LONG     *pcb      // Not used in streamout
                            )
{
  // Write the data to the disk for our example. dwCookie is used as a file
  // handle for our example.
  _lwrite ( (HFILE)dwCookie, pbBuff, cb);
  return 0;
}

/*************************************************************************

   The RTF_Save function uses the EM_STREAMOUT message to invoke
   a callback function (above) to stream the data. Our example
   simply writes the stream to a file, your code could do whatever it
   wanted to.

*************************************************************************/

BOOL RTF_Save ( HWND hWnd, HWND hRTFWnd )
{
  HFILE         hFile; // File handle
  OFSTRUCT      of;    // open file strucuture
  EDITSTREAM    es;    // The EDITSTREAM structure

  // Call the file save as dialog only if the szFileName is empty,
  // otherwise szFileName already has a valid filename in it.

  if (*szFileName || GetFileName ( hWnd, szFileName, FALSE ))
    {
    // Open the file, erasing previous contents
    hFile = OpenFile ( szFileName, &of, OF_CREATE );

    // Set up the EDITSTREAM structure
    es.dwCookie    = (DWORD)hFile;   // our file handle
    es.dwError     = 0;              // No error
    es.pfnCallback = SaveCallback;   // Use above callback

    // Engage!
    SendMessage(hRTFWnd, EM_STREAMOUT, SF_RTF, (LPARAM)&es);

    // Close the file
    _lclose ( hFile );

    return TRUE;
    }

  return FALSE;
}

/*************************************************************************

   This is the callback stream used by the RTF Edit controls
   EM_STREAMIN message. It is called repeatedly until the application
   has supplied all of the information.

*************************************************************************/

DWORD CALLBACK OpenCallback ( DWORD    dwCookie,
                              LPBYTE   pbBuff,
                              LONG     cb,
                              LONG     *pcb
                            )
{
  // Read as much data as allowed in the cb variable
  *pcb = _lread ( (HFILE)dwCookie, pbBuff, cb);
  if ( *pcb < cb )
    return 0;    // Indicate the read is complete
  else
   return *pcb;  // Read more!
}

/*************************************************************************

   The RTF_open function uses the EM_STREAMIN message to invoke
   a callback function (above) to stream the data. Our example
   simply reads the stream from a file, your code could do whatever it
   wanted to.

*************************************************************************/

BOOL RTF_Open ( HWND hWnd, HWND hRTFWnd )
{
  HFILE         hFile;  // File handle
  OFSTRUCT      of;     // open file strucuture
  EDITSTREAM    es;     // The EDITSTREAM structure

  // Get a filename to load from

  if (GetFileName ( hWnd, szFileName, TRUE ))
    {
    // Open the file for reading
    hFile = OpenFile ( szFileName, &of, OF_READ );

    if (hFile)
      {
      // Set up the EDITSTREAM structure
      es.dwCookie    = (DWORD)hFile; // our file handle
      es.dwError     = 0;            // No error
      es.pfnCallback = OpenCallback; // Use above callback

      // Make it so!
      SendMessage(hRTFWnd, EM_STREAMIN, SF_RTF, (LPARAM)&es);

      // Close the file
      _lclose ( hFile );
      }

    return TRUE;
    }

  return FALSE;
}

/*************************************************************************

   The RTF_CheckForDirtyContents function checks to see if the dirty
   bit has been set for the editt control. If so, the user is offered
   a chance to save their file. 

*************************************************************************/

BOOL RTF_CheckForDirtyContents ( HWND hWnd, HWND hRTFWnd )
{
  char szMsg[512];
  int  iRetVal;

  // Check dirty bit
  if (SendMessage ( hRTFWnd, EM_GETMODIFY, 0, 0L ))
    {
    if (*szFileName)
      wsprintf ( szMsg, 
                 "Do you wish to save the current file \"%s\"?",
                 szFileName
               );
    else
      lstrcpy ( szMsg, "Do you wish to save the current untitled file?" );

    // Get their response
    iRetVal = MessageBox ( hWnd, 
                           szMsg, 
                           "MSJ", 
                           MB_YESNOCANCEL | MB_ICONQUESTION );

    switch ( iRetVal )
      {
      case IDYES: 

        RTF_Save ( hWnd, hRTFWnd );
        return TRUE;

      case IDNO:

        return TRUE;

      case IDCANCEL:

        return FALSE;

      }
    }
  else
    {
    return TRUE;
    }
}

/*********************************************************************

   The SelectPrinter function simply uses the common dialog to
   get the parameters needed to satisfy the GetPrinterDC function.

*********************************************************************/

BOOL  SelectPrinter ( LPSTR szPrinter, 
                      HWND  hWndParent, 
                      UINT  FAR *pdmDriverExtra )
{
  PRINTDLG   pd;         // From commdlg.h
  LPDEVMODE  lpDevMode;  // From print.h
  HANDLE     hDevMode;
  LPSTR      szMem;
  int        i;

  szMem = (LPSTR)&pd;
  for ( i = 0; i < sizeof(pd); i++ ) szMem[i] = 0;

  /* Initialize the necessary pd structure members. */

  pd.lStructSize = sizeof(PRINTDLG);
  pd.hwndOwner   = hWndParent;
  pd.Flags       = PD_PRINTSETUP;

  /* Get the info we need */

  if (!PrintDlg(&pd)) return 0;

  hDevMode = pd.hDevMode;

  if (hDevMode)
    {
    lpDevMode = (LPDEVMODE)GlobalLock ( hDevMode );

    *pdmDriverExtra = lpDevMode->dmDriverExtra;

    lstrcpy ( szPrinter, lpDevMode->dmDeviceName);

    GlobalUnlock ( hDevMode );
    GlobalFree ( hDevMode );

    return TRUE;
    }
  else
    {
    *pdmDriverExtra = 0;
    return FALSE;
    }
}

/**************************************************************

    The GetDCHook function is the dialog procedure for the
    print common dialog. This function's sole purpose in life is
    to press the OK button instantly. This causes the dialog to never
    appear, yet we can get a printer DC back from it.

**************************************************************/

BOOL CALLBACK GetDCHook ( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
  switch ( msg )
    {
    case WM_INITDIALOG:

      PostMessage ( hDlg, WM_COMMAND, IDOK, 0L );
      return FALSE;

    default:

      return FALSE;
    }
}

/***********************************************************

  This GetPrinterDC function calls the print common dialog
  and uses the above hook to instantly press the OK button,
  which ensures the dialog never appears. By using the PD_RETURNDC
  flag, we can get back a printer DC for whatever print the user
  selcted without having to do any gross stuff.

***********************************************************/

HDC GetPrinterDC ( LPSTR szPrinter, 
                   UINT  dmDriverExtra, 
                   HWND  hWndParent )
{
  PRINTDLG   pd;         // From commdlg.h
  LPDEVMODE  lpDevMode;  // From print.h
  HANDLE     hDevMode;
  LPSTR      szMem;
  int        i;

  hDevMode = GlobalAlloc ( GHND, sizeof(DEVMODE)+(LONG)dmDriverExtra);
  if (!hDevMode) return (HDC)0;

  lpDevMode = (LPDEVMODE)GlobalLock ( hDevMode );

  lstrcpy ( lpDevMode->dmDeviceName, szPrinter);
  lpDevMode->dmSize        = sizeof(DEVMODE);
  lpDevMode->dmSpecVersion = 0x030A;
  lpDevMode->dmDriverExtra = dmDriverExtra;

  GlobalUnlock ( hDevMode );
   
  szMem = (LPSTR)&pd;
  for ( i = 0; i < sizeof(pd); i++ ) szMem[i] = 0;

  /* Initialize the necessary pd structure members. */

  pd.lStructSize  = sizeof(PRINTDLG);
  pd.hwndOwner    = hWndParent;
  pd.Flags        = PD_RETURNDC | PD_ENABLEPRINTHOOK;
  pd.hDevMode     = hDevMode;
  pd.lpfnPrintHook= GetDCHook;

  if (!PrintDlg(&pd)) return 0;

  GlobalFree ( hDevMode );

  return pd.hDC;
}

/***********************************************************

  The RTF_Print function gets a printer DC and then prints one
  page at a time. 

***********************************************************/

void RTF_Print ( HWND hWnd, HWND hRTFWnd )
{
  HDC          hPrinterDC;
  FORMATRANGE  fr;
  DOCINFO      di;
  LONG         lTextLength, lTextPrinted;

  // Get a printer DC
  if (*szPrinter || SelectPrinter ( szPrinter, hWnd, &dmDriverExtra ))
    {
    hPrinterDC = GetPrinterDC ( szPrinter, dmDriverExtra, hWnd );
    if (!hPrinterDC)
      {
      MessageBox ( hWnd, "CreateDC Failed", "RTF", MB_OK );
      return;
      }
    }
  else
    {
    MessageBox ( hWnd, "No printer selected", "RTF", MB_OK );
    return;
    }

  // At this point, we have a printer DC

  // Step 1: Identify the page size

  // Rendering to the same DC we are measuring
  fr.hdc = fr.hdcTarget = hPrinterDC;

  // Set up a 6x9" printing area
  fr.rc.left    = fr.rc.top = 0;
  fr.rc.right   = 6*1440;  // 6" wide
  fr.rc.bottom  = 9*1440; // 9" down
  fr.rcPage     = fr.rc;

  // Default the range of text to print as the entire document
  fr.chrg.cpMin = 0;
  fr.chrg.cpMax = -1;

  // Set up the print job (standard printing stuff here)
  di.cbSize = sizeof ( DOCINFO );
  if (*szFileName)
    di.lpszDocName = szFileName;
  else
    di.lpszDocName = "(Untitled)";
  di.lpszOutput = NULL;

  // Ensure the printer DC is in MM_TEXT mode
  SetMapMode ( hPrinterDC, MM_TEXT );

  StartDoc ( hPrinterDC, &di );
  StartPage ( hPrinterDC );

  // Find out real size of document in characters
  lTextLength = SendMessage ( hRTFWnd, WM_GETTEXTLENGTH, 0, 0 );

  do
    {  
    // Print as mush as will fit on a page. The return value
    // is the index of the first character on the next page.
    lTextPrinted = SendMessage ( hRTFWnd, EM_FORMATRANGE, TRUE, (LPARAM)&fr );

    // If there is more text to print, spit this page from the printer
    // and start another one.
    if (lTextPrinted < lTextLength)
      {
      EndPage ( hPrinterDC );
      StartPage ( hPrinterDC );

      // Adjust the range of characters to print to start at the
      // character of the next page
      fr.chrg.cpMin = lTextPrinted;
      fr.chrg.cpMax = -1;
      }
    }
  while (lTextPrinted < lTextLength);

  // Restore the RTF Edit control to its original, upright position
  SendMessage ( hRTFWnd, EM_FORMATRANGE, TRUE, (LPARAM)NULL );

  // Spit out last page
  EndPage ( hPrinterDC );
  EndDoc ( hPrinterDC );

  // Remove that pesky DC
  DeleteDC ( hPrinterDC );
}

//************************* NEW CODE START ************************   

/*****************************************************************

   The TOOLBAR_DetermineToolbarButtonState function will 
   check/uncheck/waffle a toolbr button.

*****************************************************************/

void TOOLBAR_DetermineToolbarButtonState (HWND hTBWnd, UINT iID, UINT iState)
{
  switch ( iState )
    {
    // For YES or NO values, remove the waffle (INDETERMINATE) style
    // from the button, and then check or uncheck it.
    case CHAR_ATTRIB_YES:
    case CHAR_ATTRIB_NO :

      SendMessage (hTBWnd, TB_INDETERMINATE, iID, MAKELONG(FALSE,0) );
      SendMessage (hTBWnd, TB_CHECKBUTTON, iID,
                   MAKELONG((CHAR_ATTRIB_YES==iState),0) );
      break;

    // Otherwise, set the waffle (INDETERMINATE) style
    default: // CHAR_ATTRIB_WAFFLE

      SendMessage (hTBWnd, TB_INDETERMINATE, iID, MAKELONG(TRUE,0) );
      break;
    }
}

/*****************************************************************
  
   The RTF_GetCurrentFont function filld in the szFont string with
   the facename of the current selection in the RTF Edit control.

*****************************************************************/

void RTF_GetCurrentFont ( HWND hRTFWnd, LPSTR szFont )
{
    CHARFORMAT cf;                // This is defined in RICHEDIT.H

    // Get the font formatting status into the CHARFORMAT strucuture,
    // use the selected text. wParam is TRUE to indicate the
    // selected text, FALSE for the first character in the
    // RTF Edit control.
    SendMessage(hRTFWnd, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

    if (cf.dwMask & CFM_FACE)
      lstrcpy ( szFont, cf.szFaceName );
    else
      *szFont = 0;

}

/*****************************************************************

  The RTF_SetCurrentFont function sets the font of the current
  selection of the RTF Edit control to the font pointed to by
  the szFont string.

*****************************************************************/

void RTF_SetCurrentFont ( HWND hRTFWnd, LPSTR szFont )
{
    CHARFORMAT cf;                // This is defined in RICHEDIT.H

    // Get the font formatting status into the CHARFORMAT strucuture,
    // use the selected text. wParam is TRUE to indicate the
    // selected text, FALSE for the first character in the
    // RTF Edit control.
    SendMessage(hRTFWnd, EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);

    cf.dwMask = CFM_FACE;

    lstrcpy ( cf.szFaceName, szFont );

    SendMessage(hRTFWnd, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
}

/***********************************************************

  The RTF_UpdateMenuState function will check/uncheck the
  Bold, italic, and Underline menu options if the current text
  selections have those attributes. It will also update the
  toolbar.

***********************************************************/

void RTF_UpdateMenuState ( HWND hWndRTF, HWND hWnd )
{
  // Values of CHAR_ATTRIB_YES, CHAR_ATTRIB_NO, or CHAR_ATTRIB_WAFFLE
  int       iBold, iItalic, iUnderline;
  
  HMENU     hMenu;               // Menu to munge
  char      szFont[LF_FACESIZE]; // Current font facename
  WPARAM    wFontIndex;          // Index into combobox

  // Get the attributes for each type of attrib
  iBold = RTF_GetCharAttribute ( hRTFWnd, 
                                 CFM_BOLD, CFE_BOLD );
  iItalic = RTF_GetCharAttribute ( hRTFWnd, 
                                   CFM_ITALIC, CFE_ITALIC );
  iUnderline = RTF_GetCharAttribute ( hRTFWnd, 
                                      CFM_UNDERLINE, CFE_UNDERLINE );
  // And check/uncheck the menus
  hMenu = GetMenu ( hWnd );
  CheckMenuItem (hMenu, IDM_BOLD,      MF_BYCOMMAND |
                 CHAR_ATTRIB_YES==iBold      ? MF_CHECKED : MF_UNCHECKED );
  CheckMenuItem (hMenu, IDM_ITALIC,    MF_BYCOMMAND |
                 CHAR_ATTRIB_YES==iItalic    ? MF_CHECKED : MF_UNCHECKED );
  CheckMenuItem (hMenu, IDM_UNDERLINE, MF_BYCOMMAND |
                 CHAR_ATTRIB_YES==iUnderline ? MF_CHECKED : MF_UNCHECKED );

  // And check/uncheck/waffle the toolbar buttons
  TOOLBAR_DetermineToolbarButtonState (hTBWnd, IDM_BOLD,      iBold     );
  TOOLBAR_DetermineToolbarButtonState (hTBWnd, IDM_ITALIC,    iItalic    );
  TOOLBAR_DetermineToolbarButtonState (hTBWnd, IDM_UNDERLINE, iUnderline);

  // Get the facename of the current font
  RTF_GetCurrentFont ( hRTFWnd, szFont );

  // Find and select it in the toolbar
  wFontIndex = (WPARAM)SendMessage ( hComboBox, CB_FINDSTRING, (WPARAM)-1, (LPARAM)szFont );
  SendMessage ( hComboBox, CB_SETCURSEL, wFontIndex, 0L );
}

/***********************************************************

  The TOOLTIP_GetText function is called in response to the
  WM_NOTIFY:TTN_NEEDTEXT message. We need to fill in lpszText to
  point to a string to display in the tooltip.

***********************************************************/

void  TOOLTIP_GetText ( LPTOOLTIPTEXT lpttt )
{
  static char  szText[128];

  lpttt->lpszText = szText; // Point to a buffer

  // Grab a string from the stringtable. The
  // lpttt->hdr.idFrom field contains the ID of the button
  // that needs text. The TOOLTIPOFFSET is a constant that
  // is used in the stringtable in this program's RC file.

  LoadString(hInst, 
             lpttt->hdr.idFrom + TOOLTIPOFFSET,
             szText,
             sizeof(szText));
}

/***********************************************************

   This function is called from the Handle_WM_MENSELECT function
   below. If idMsg is not -1, the status bar is set to simple
   mode and the text from the string table with that id is displayed
   in it. if idMsg is -1, the status bar is returned back to
   normal mode.

***********************************************************/

void STATUS_DisplayStatusMenuHelp ( HWND hStatusWnd, UINT idMsg )
{
  char sz[128];

  if ( -1 != idMsg )
    {
    // Set status bar to simple mode
    SendMessage ( hStatusWnd, SB_SIMPLE, TRUE, 0 );

    if (0 != idMsg)
       {
       // Get the string from the stringtable
       LoadString(hInst, idMsg, sz, sizeof(sz));
       // Set the text
       SendMessage ( hStatusWnd, SB_SETTEXT, 255, (LPARAM)sz );
       }
    }
  else
    {
    // Set the status bar back to normal mode
    SendMessage ( hStatusWnd, SB_SIMPLE, FALSE, 0 );
    }

}

/***********************************************************

  The STATUS_DisplayStatusInformation fills in the pane
  of the status bar that indicates the row and column of the
  caret inside the RTF edit control

***********************************************************/

void STATUS_DisplayStatusInformation ( HWND hRTFWnd, HWND hStatusWnd )
{
  char      sz[256];
  int       iRow, iCol;
  CHARRANGE cr;

  // Find out the position of the caret (if a selection is set,
  // use the endpoint of the selection).
  SendMessage ( hRTFWnd, EM_EXGETSEL, 0, (LPARAM)&cr );

  // Figure the row and column
  iRow = (int)SendMessage ( hRTFWnd, EM_EXLINEFROMCHAR, 0, (LPARAM)(cr.cpMax) );
  iCol = (int)
           (
           cr.cpMax - (int)SendMessage ( hRTFWnd, EM_LINEINDEX, iRow, 0 )
           );

  // Format the string
  wsprintf ( sz, "Ln %d    Col %d", iRow+1, iCol+1 );

  // Set the text in the second pane (Pane #1)
  SendMessage ( hStatusWnd, SB_SETTEXT, 1, (LPARAM)sz );
}

#define GET_WM_MENUSELECT_FLAGS(uParam, lParam) ((UINT)HIWORD(uParam))
#define GET_WM_MENUSELECT_CMD(uParam, lParam)   ((UINT)LOWORD(uParam))
#define GET_WM_MENUSELECT_HMENU(uParam, lParam) ((HMENU)lParam)

/***********************************************************
 
   The Handle_WM_MENSELECT function below is a pared down
   cheezy sample to figure out the ID of the currently selected
   menu. It returns 0 if a popup menu is selected, -1 of no menu
   is selected (i.e. closed), and a positive nonzero value
   if a menu item is selected.

***********************************************************/

void Handle_WM_MENSELECT ( HWND hStatusWnd, UINT message, WPARAM uParam, LPARAM lParam )
{
    static char szBuffer[128];
    UINT   nStringID = 0;

    UINT   fuFlags = GET_WM_MENUSELECT_FLAGS(uParam, lParam) & 0xffff;
    UINT   uCmd    = GET_WM_MENUSELECT_CMD(uParam, lParam);
    HMENU  hMenu   = GET_WM_MENUSELECT_HMENU(uParam, lParam);

    szBuffer[0] = 0;                            // First reset the buffer

    nStringID = 0;

    if (fuFlags == 0xffff && hMenu == NULL)     // Menu has been closed
        nStringID = (UINT)-1;

    else if (fuFlags & MFT_SEPARATOR)           // Ignore separators
        nStringID = 0;

    else if (fuFlags & MF_POPUP)                // Popup menu
        {
        if (fuFlags & MF_SYSMENU)               // System menu
            nStringID = 0;
        }  // for MF_POPUP
    else                                        // Must be a command item
        {
        nStringID = uCmd;               // String ID == Command ID
        }

  // Update the status bar
  STATUS_DisplayStatusMenuHelp ( hStatusWnd, nStringID );
}

//*********************** NEW CODE END *******************


